"""
cogwatch - Automatic hot-reloading for your discord.py / nextcord command files.
"""

__title__ = "cogwatch"
__author__ = "Rob Wagner <rob@sombia.com>"
__license__ = "MIT"
__copyright__ = "Copyright 2020-2022 Rob Wagner"
__version__ = "3.1.0"

from cogwatch.cogwatch import Watcher, watch
